import { useMemo, ReactNode, ReactElement } from 'react';
import { NodeAttribute, QueryGraphEdges, SchemaReactflowEntityNode, handleDataFromReactflowToDataId, toHandleId } from '../../model';
import { Position } from 'reactflow';
import styles from './pilldropdown.module.scss';
import { FilterHandle } from '../FilterHandle';
import { Abc, CalendarToday, Map, Numbers, Place, QuestionMarkOutlined } from '@mui/icons-material';
import Icon from '@graphpolaris/shared/lib/components/icon';

type PillDropdownProps = {
  node: SchemaReactflowEntityNode;
  attributes: NodeAttribute[];
  attributeEdges: (QueryGraphEdges | undefined)[];
  hovered: boolean;
  handleBeingDraggedIdx: number;
  onHandleMouseDown: (attribute: NodeAttribute, i: number, event: React.MouseEvent) => void;
};

type IconMapType = {
  [key: string]: ReactElement;
};

const IconMap: IconMapType = {
  temporal: <CalendarToday />,
  spatial: <Map />,
  numerical: <Numbers />,
  categorical: <Abc />,
  default: <Place />,
};

export const PillDropdown = (props: PillDropdownProps) => {
  const forceOpen = false;
  const openNumbers: number[] = useMemo(() => {
    if (forceOpen || props.hovered) return props.attributes.map((_, i) => i);

    //collect connected nodes
    const connected = props.attributes
      .map((attribute) => {
        const nodeID = toHandleId(handleDataFromReactflowToDataId(props.node, attribute));
        return props.attributeEdges.some((edge) => edge?.sourceHandleData && toHandleId(edge?.sourceHandleData) === nodeID);
      })
      .map((connected, i) => (connected ? i : -1))
      .filter((i) => i !== -1);

    // add to it the node being dragged
    if (props.handleBeingDraggedIdx !== -1 && !connected.includes(props.handleBeingDraggedIdx)) connected.push(props.handleBeingDraggedIdx);
    return connected.sort();
  }, [props]);

  return (
    <>
      <div className={'divide-y divide-secondary-100' + (openNumbers.length > 0 ? 'animate-openmenu ' : 'animate-closemenu ')}>
        {openNumbers.map((i) => {
          const attribute = props.attributes[i];
          if (attribute.handleData.attributeName === undefined) {
            throw new Error('attribute.handleData.attributeName is undefined');
          }

          return (
            <div
              className="px-2 py-1 bg-white flex justify-between items-center"
              key={(attribute.handleData.attributeName || '') + i}
              onMouseDown={(event: React.MouseEvent) => {
                props.onHandleMouseDown(attribute, i, event);
              }}
            >
              <p>{attribute.handleData.attributeName}</p>
              {attribute.handleData?.attributeDimension && (
                // <div className="!text-xs text-secondary-500">{IconMap[attribute.handleData.attributeDimension]}</div>
                // <div className="!text-xs text-secondary-500">
                <Icon component={IconMap[attribute.handleData.attributeDimension]} size={16} />
                // </div>
              )}
              <FilterHandle
                handle={handleDataFromReactflowToDataId(props.node, attribute)}
                type="source"
                position={Position.Right}
                className={styles.handle + ' ' + '!top-auto mt-2 !right-[0.5rem] bg-secondary-500'}
              ></FilterHandle>
            </div>
          );
        })}
      </div>
    </>
  );
};
