import React, { useState, useEffect } from 'react';
import { VISComponentType, VisualizationConfiguration } from './types';
import { removeVisualization, reorderVisState, setActiveVisualization, updateVisualization } from '../data-access/store/visualizationSlice';
import {
  useAppDispatch,
  useGraphQueryResult,
  useGraphQueryResultMeta,
  useML,
  useSchemaGraph,
  useSessionCache,
  useVisualization,
} from '../data-access';
import { FormActions, FormBody, FormCard, FormDiv, FormHBar, FormTitle } from '../components';

export const Visualizations: Record<string, Function> = {
  TableVis: () => import('./visualizations/tablevis/tablevis'),
  PaohVis: () => import('./visualizations/paohvis/paohvis'),
  RawJSONVis: () => import('./visualizations/rawjsonvis/rawjsonvis'),
  NodeLinkVis: () => import('./visualizations/nodelinkvis/nodelinkvis'),
  // MapVis: () => import('./visualizations/mapvis/mapvis'),
  MatrixVis: () => import('./visualizations/matrixvis/matrixvis'),
};

const INITIAL_VISUALIZATIONS = ['NodeLinkVis', 'PaohVis'];

export const useVisualizationManager = () => {
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const ml = useML();
  const schema = useSchemaGraph();
  const graphQueryResult = useGraphQueryResult();
  const meta = useGraphQueryResultMeta();
  const { active, visualizations } = useVisualization();

  const [configuration, setConfiguration] = useState<any>();
  const [visualization, setVisualization] = useState<VISComponentType>();
  const [hoverItem, setHoverItem] = useState<any>();
  const [selected, setSelected] = useState<any>();
  const activeType = visualizations[active]?.displayName;

  const firstRender = !active && !session.currentSaveState;
  const tabs = Object.keys(visualizations).length ? Object.keys(visualizations) : INITIAL_VISUALIZATIONS;

  useEffect(() => {
    loadVisualization();
  }, [active]);

  const loadVisualization = async () => {
    if (active && Visualizations[active]) {
      const componentModule = await Visualizations[active]();
      const component = componentModule.default;

      if (!(active in Object.keys(visualizations))) {
        // Visualization doesn't yet exist so add its configuration
        const configuration = component.configuration;
        updateSettings(configuration);
      }

      setConfiguration(visualizations[active]);
      setVisualization(component);
    }
  };

  useEffect(() => {
    if (firstRender) {
      dispatch(setActiveVisualization(INITIAL_VISUALIZATIONS[0]));
    }
  }, [active, dispatch]);

  const changeActive = (id: string) => {
    dispatch(setActiveVisualization(id));
  };

  const deleteVisualization = (id: string) => {
    dispatch(removeVisualization(id));
  };

  const reorderVisualizations = ({ draggedVisId, dropVisId }: { draggedVisId: string; dropVisId: string }) => {
    const settingsCopy = { ...visualizations };
    const keys = Object.keys(settingsCopy);
    const draggedIndex = keys.indexOf(draggedVisId);
    const dropIndex = keys.indexOf(dropVisId);

    if (draggedIndex !== -1 && dropIndex !== -1) {
      keys.splice(dropIndex, 0, draggedVisId);
      const newSettings: { [id: string]: VisualizationConfiguration } = {};
      keys.forEach((key) => {
        newSettings[key] = settingsCopy[key];
      });

      dispatch(reorderVisState(newSettings));
    }
  };

  const handleHover = (item: any) => {
    setHoverItem(item);
  };

  const handleSelect = (item: any) => {
    setSelected(item);
  };

  const updateSettings = (newSettings: any) => {
    if (active) {
      const updatedSettings = { ...configuration, ...newSettings };
      setConfiguration(updatedSettings);
      dispatch(updateVisualization({ id: active, settings: updatedSettings }));
    }
  };

  const renderSettings = () => {
    return (
      visualization?.settings &&
      configuration && <visualization.settings configuration={configuration} graph={meta} updateSettings={updateSettings} />
    );
  };

  const renderComponent = () => {
    return (
      visualization?.component &&
      configuration && (
        <visualization.component
          data={graphQueryResult}
          schema={schema}
          ml={ml}
          configuration={configuration}
          dispatch={dispatch}
          handleHover={handleHover}
          handleSelect={handleSelect}
        />
      )
    );
  };

  return {
    renderComponent,
    renderSettings,
    active,
    activeType,
    tabs,
    changeActive,
    reorderVisualizations,
    deleteVisualization,
  };
};
